﻿#include "AppDelegate.h"
#include "MagicScene.h"

#include "mp_cocos.h"
#include "platform_cocos.h"

USING_NS_CC;

static cocos2d::Size designResolutionSize = cocos2d::Size(480, 320);
static cocos2d::Size smallResolutionSize = cocos2d::Size(480, 320);
static cocos2d::Size mediumResolutionSize = cocos2d::Size(1024, 768);
static cocos2d::Size largeResolutionSize = cocos2d::Size(2048, 1536);

AppDelegate::AppDelegate()
{
	device=NULL;
}

AppDelegate::~AppDelegate() 
{
	if (device)
	{
		device->Destroy();
		delete device;
		device=NULL;
	}

	MP_Manager& MP=MP_Manager::GetInstance();
	MP.Destroy();
}

//if you want a different context,just modify the value of glContextAttrs
//it will takes effect on all platforms
void AppDelegate::initGLContextAttrs()
{
    //set OpenGL context attributions,now can only set six attributions:
    //red,green,blue,alpha,depth,stencil
    GLContextAttrs glContextAttrs = {8, 8, 8, 8, 24, 8};

    GLView::setGLContextAttrs(glContextAttrs);
}

// If you want to use packages manager to install more packages, 
// don't modify or remove this function
static int register_all_packages()
{
    return 0; //flag for packages manager
}

bool AppDelegate::applicationDidFinishLaunching() {
    // initialize director
    auto director = Director::getInstance();
    auto glview = director->getOpenGLView();
    if(!glview) {
        glview = GLViewImpl::create("Magic Particles (www.astralax.com) wrapper for Cocos2dx");
        director->setOpenGLView(glview);
    }

    // turn on display FPS
    director->setDisplayStats(true);

    // set FPS. the default value is 1.0/60 if you don't call this
    director->setAnimationInterval(1.f / 60.f);

    // Set the design resolution
    glview->setDesignResolutionSize(designResolutionSize.width, designResolutionSize.height, ResolutionPolicy::NO_BORDER);
    Size frameSize = glview->getFrameSize();
    // if the frame's height is larger than the height of medium size.
    if (frameSize.height > mediumResolutionSize.height)
    {        
        director->setContentScaleFactor(MIN(largeResolutionSize.height/designResolutionSize.height, largeResolutionSize.width/designResolutionSize.width));
    }
    // if the frame's height is larger than the height of small size.
    else if (frameSize.height > smallResolutionSize.height)
    {        
        director->setContentScaleFactor(MIN(mediumResolutionSize.height/designResolutionSize.height, mediumResolutionSize.width/designResolutionSize.width));
    }
    // if the frame's height is smaller than the height of medium size.
    else
    {        
        director->setContentScaleFactor(MIN(smallResolutionSize.height/designResolutionSize.height, smallResolutionSize.width/designResolutionSize.width));
    }

    register_all_packages();

	// -----------------------------------------------------------------------------------------

	cocos2d::Size winSize = Director::getInstance()->getWinSizeInPixels();

	int client_wi=winSize.width;
	int client_he=winSize.height;

	device=new MP_Device_Cocos(client_wi, client_he);
	device->Create();

	MP_Manager& MP=MP_Manager::GetInstance();

	MP_Platform* platform=new MP_Platform_COCOS;
	#ifdef MAGIC_3D
	MAGIC_AXIS_ENUM axis=MAGIC_pXpYnZ;
	#else
	MAGIC_AXIS_ENUM axis=MAGIC_pXnY;
	#endif

	#ifdef SHADER_WRAP
	bool filters[MAGIC_RENDER_STATE__MAX];
	for (int i=0;i<MAGIC_RENDER_STATE__MAX;i++)
		filters[i]=false;
	filters[MAGIC_RENDER_STATE_BLENDING]=true;
	filters[MAGIC_RENDER_STATE_TEXTURE]=true;
	filters[MAGIC_RENDER_STATE_ADDRESS_U]=true;
	filters[MAGIC_RENDER_STATE_ADDRESS_V]=true;
	filters[MAGIC_RENDER_STATE_ZENABLE]=true;
	filters[MAGIC_RENDER_STATE_ZWRITE]=true;
	#ifndef SHADER_ALPHATEST_WRAP
	filters[MAGIC_RENDER_STATE_ALPHATEST_INIT]=true;
	filters[MAGIC_RENDER_STATE_ALPHATEST]=true;
	#endif
	#else
	bool* filters=NULL;
	#endif

	MP.Initialization(filters, true, axis, platform, MAGIC_INTERPOLATION_ENABLE, MAGIC_CHANGE_EMITTER_DEFAULT, 1024, 1024, 1, 1.f, 0.1f, true);

	MP.LoadAllEmitters();

	MP.RefreshAtlas();

	MP.CloseFiles();

	MP.Stop();

	#ifndef MAGIC_3D
	// eng: locate emitters the same as editor
	// rus: расставляем эмиттеры также, как они стояли в редакторе
	MP_Emitter* emitter=MP.GetFirstEmitter();
	while (emitter)
	{
		Magic_CorrectEmitterPosition(emitter->GetEmitter(), client_wi, client_he);
		emitter=emitter->GetNext();
	}
	#endif

	// -----------------------------------------------------------------------------------------
	
    // create a scene. it's an autorelease object
    auto scene = MagicScene::createScene();

    // run
    director->runWithScene(scene);

    return true;
}

// This function will be called when the app is inactive. When comes a phone call,it's be invoked too
void AppDelegate::applicationDidEnterBackground() {
    Director::getInstance()->stopAnimation();

    // if you use SimpleAudioEngine, it must be pause
    // SimpleAudioEngine::getInstance()->pauseBackgroundMusic();
}

// this function will be called when the app is active again
void AppDelegate::applicationWillEnterForeground() {
    Director::getInstance()->startAnimation();

    // if you use SimpleAudioEngine, it must resume here
    // SimpleAudioEngine::getInstance()->resumeBackgroundMusic();
}
